/* --COPYRIGHT--,BSD
 * Copyright (c) 2016,2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  stepper.c - Definition file for functions related to stepper motion control
 *  DRV8434S_Example FIRMWARE
 *  10/28/2020
 *  updated 01/20/2022
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <stepper.h>

/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gStepper_Obj,".stpVarSection");
volatile Stepper_Driving_Obj_t gStepper_Obj;

/*****************************************************************************
 * Define Global Variables in stepper.h
 *****************************************************************************/
//#define DISABLE_STALL_DURING_ACCEL
/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// TimerA Initialization for PWM
void stepper_Init(void)
{
    enable_TAInterrupt = true;

    gStepper_Obj.StallSampleTime = TimebetweenSamples;
    capture_stall_info_cnt = 0;
    set_ENABLE_lo;
    // configure timer A
    TACTL = TASSEL_2 | ID_0 | MC_0 | TACLR;                // use 8MHz clock

    // configure timer B
    // select SMCLK, divide by 8, stop
    TBCTL = TBSSEL_2 | ID_3 | MC_0 | TBCLR;
    P1SEL |= STEP;                                         // STEP pin GPIO select

    // Set Default GUI Variables Values
    SetDefaultGUIVals();

    gStepper_Obj.gDIR = false;

    // Load GPIO Values
    UpdateDeviceControls();
    UpdateGUIControls();

    // set boolean defaults
    LastCommandWasRunSteps = false;
    MoveSteps = false;
    UpdateSteppingRateTMR = false;
    Use2MHzTimerClk = false;
    temp_reciprocation = false;
    RecoverFromStall = false;
    stall_learn_number_of_steps = 0;
    stall_learn_microstep_divider = 0;
    last_stall_state = DISABLED;
    SPI_STEP_DIR_CMD = 0;
    SteppedOnce = 3;
    ok_to_start_timer = true;
}

void SetDefaultGUIVals()
{
    last_reciprocation_state = 0;
    gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
    gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
    gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
    gStepper_Obj.SPI_STEP_DIR_EN = false;
    gStepper_Obj.STARTING_SPEED = DEFAULT_START_SPEED;     // initial speed in pulses per second when starting motor
    gStepper_Obj.TARGET_SPEED = DEFAULT_TARGET_SPEED;      // speed of motor once running
    gStepper_Obj.ACCEL_RATE = DEFAULT_ACCEL_RATE;          // acceleration rate from starting speed to target speed
    gStepper_Obj.STOPPING_SPEED = DEFAULT_STOPPING_SPEED;  // speed to stop motor once deceleration has reached this point
    gStepper_Obj.NUM_STEPS = DEFAULT_NUM_STEPS;            // number of steps to move when manually stepping
    gStepper_Obj.STEPS_2_STOP = DEFAULT_STEPS_2_STOP;      // number of steps used to stop the motor when manually stepping
    gStepper_Obj.IFS = DEFAULT_IFS;
    IFS_previous = DEFAULT_IFS + 1;
    gStepper_Obj.REVERSE_ON_STALL = false;
    gStepper_Obj.RUN_STEPPER = false;                      // Auto run enabled
    gStepper_Obj.MANUAL_ADVANCE = false;                   // Manually advance motor
    gStepper_Obj.RECIPROCATION = false;
    gStepper_Obj.StallState = DISABLED;
    gStepper_Obj.Active_STL_DET = false;
    gStepper_Obj.DISABLE_STALL_ACCEL = true;
}

// Routine to communicate between GUI and device
// reads GUI variables and updates the GPIOs
void UpdateDeviceControls()
{
    // toggle status if count is reached
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_status_led;
        LED_TOGGLE_CNT = 0;
    }

    // From GUI to MCU
    // Update the GPIO Pins from Global Variables
    if (read_nFAULT_pin == 0)                              // Fault Condition
    {
        gDRV8434_Obj.nFault = 0;
        // read the FAULT_STATUS Register for GUI display
        gDeviceSpiReg.fault_status_reg = SPI_readRegister(
                (uint8_t) SPI_REG_FAULT_STATUS);
    }
    else
        gDRV8434_Obj.nFault = 1;

    if (gDRV8434_Obj.nSleep == 1)                          // Asleep and Awake widget control
    {
        set_nSLEEP_hi;
    _delay_cycles(16000);                                  // 1 ms wake up delay
    }
    else
    {
        gStepper_Obj.StallState == DISABLED;
        last_stall_state = DISABLED;
        _delay_cycles(1600000);                            // 100 ms delay
        gStepper_Obj.Active_STL_DET = false;
        gStepper_Obj.REVERSE_ON_STALL = false;
        gStepper_Obj.DISABLE_STALL_ACCEL = true;
        set_nSLEEP_lo;
    }

    // DIR
    if (gStepper_Obj.gDIR == 1  && nextDIR == 1)
    {
        if (gStepper_Obj.MANUAL_ADVANCE == false && gStepper_Obj.RUN_STEPPER == true)
        {
            ok_to_start_timer = false;
            gStepper_Obj.RUN_STEPPER = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
            __delay_cycles(16000);                         // 1ms delay
            set_DIR_hi;
            nextDIR = 0;
            gStepper_Obj.RUN_STEPPER = true;
            ok_to_start_timer = true;
        }
        else
        {
            set_DIR_hi;
            nextDIR = 0;
        }
    }
    else if (gStepper_Obj.gDIR == 0  && nextDIR == 0)
    {
        if (gStepper_Obj.MANUAL_ADVANCE == false && gStepper_Obj.RUN_STEPPER == true)
        {
            ok_to_start_timer = false;
            gStepper_Obj.RUN_STEPPER = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
            __delay_cycles(16000);                         // 1ms delay
            set_DIR_lo;
            nextDIR = 1;
            gStepper_Obj.RUN_STEPPER = true;
            ok_to_start_timer = true;
        }
        else
        {
            set_DIR_lo;
            nextDIR = 1;
        }
    }

    // ENABLE  -- set port pin high to enable device
    if (gDRV8434_Obj.gDRVEN == 1)
    {
        set_ENABLE_hi;
    }
    else
        set_ENABLE_lo;

    // Set EN_OUT register field to match DRV enable pin state
    if (gDRV8434_Obj.nSleep == 1)
    {
        if (P1OUT & ENABLE)
        {
            if ((gDeviceSpiReg.ctrl2_reg & EN_OUT_MASK) == 0)
            {
                gDeviceSpiReg.ctrl2_reg |= EN_OUT_MASK;
                SPI_writeRegister(SPI_REG_CTRL2, gDeviceSpiReg.ctrl2_reg);
            }
        }
        else
        {
            if ((gDeviceSpiReg.ctrl2_reg & EN_OUT_MASK))
            {
                gDeviceSpiReg.ctrl2_reg &= ~(EN_OUT_MASK);
                SPI_writeRegister(SPI_REG_CTRL2, gDeviceSpiReg.ctrl2_reg);
            }
        }
    }
}

// routine to communicate between device and GUI
// reads GPIOs and updates the GUI
void UpdateGUIControls()
{
    // Enable Motor control buttons
    if (gDRV8434_Obj.gDRVEN == 0 || gDRV8434_Obj.nSleep == 0)
    {
        // when disabled reset the indicators in the motor operation
        gStepper_Obj.RUN_STEPPER = false;
        gStepper_Obj.MANUAL_ADVANCE = false;
        gStepper_Obj.RECIPROCATION = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
        // stop the timer
        TACTL &= ~MC_3;                                    // Stop timer by setting mc_0
        TACCTL1 &= ~(OUTMOD_7 + OUT);                      // Output low
        TACTL |= TACLR;
        // clear temp_reciprocation and MoveSteps so next movement is correct
        temp_reciprocation = false;                        // reset reciprocation
        MoveSteps = false;
        gStepper_Obj.AccelerateState = IDLE;               // set AccelerateState to idle when
        // sleeping or disabled
    }
    if (gStepper_Obj.IFS != IFS_previous)
    {
        // 1638 is the multiplier based on the max full-scale current.
        // DRV8434 has a max current drive of 2.5A
        // 1638 * 2.5 = 4095 (max 12-bit DAC value)
        DAC12_1DAT = 1638 * gStepper_Obj.IFS;
        IFS_previous = gStepper_Obj.IFS;
        _nop();
    }
}

// routine to determine if any changes are in motor operation are required
// and pass the information to the motor state machine
void UpdateSpeed()
{
    StartingSpeed = (unsigned long) gStepper_Obj.STARTING_SPEED << use_one_edge; // Configure the starting speed
    // if StartingSpeed is < 150 change SMCLK_FREQ to 2000000
    if ((gStepper_Obj.STARTING_SPEED < 150)
            || (gStepper_Obj.STOPPING_SPEED < 150))
    // set Timer CLK to 2MHz
    // timer accuracy is lost by reducing the clock frequency
    // and modify results of SMCLK/Starting using SMCLK_FREQ_2MHz
        Use2MHzTimerClk = true;

    else
        Use2MHzTimerClk = false;
    //
    StoppingSpeed = (unsigned long) gStepper_Obj.STOPPING_SPEED << use_one_edge; // Configure the stopping speed
    StepsToMove = (unsigned long) gStepper_Obj.NUM_STEPS;
    StepsToStop = (unsigned long) gStepper_Obj.STEPS_2_STOP << use_one_edge;     // adjust for both edges
    DesiredTargetSpeed = (unsigned long) gStepper_Obj.TARGET_SPEED
            << use_one_edge;                                                     // Configure the target speed
    AccelRate = (unsigned long) gStepper_Obj.ACCEL_RATE << use_one_edge;         // Configure the acceleration rate here

    DetermineMotorMovement();                                                    // Once type of motor movement is determined
    MotorStateMachine();                                                         // Command the motor to move

    // enable or disable Timer A interrupt based on signal
    if (enable_TAInterrupt == true)
        TACCTL0 |= CCIE;
    else
        TACCTL0 &= ~CCIE;
}

//  Routine to determine desired motor operation
//  direction, acceleration, deceleration, steps to move
//  reciprocation
void DetermineMotorMovement()
{
    // Disable stall detection if Initiate Stall is disabled and EN_STL in CTRL5 is set
    if ((gStepper_Obj.StallState == 0) && (gDeviceSpiReg.ctrl5_reg & EN_STL_MASK))  // Disable stall detection if Initiate Stall is disabled
    {
        SPI_writeRegister(SPI_REG_CTRL5, (gDeviceSpiReg.ctrl5_reg & DIS_STL_MASK)); // Disable Stall detection
    }

    // calculate steps to target and steps to stop prior to starting motor
    if ((gStepper_Obj.RUN_STEPPER == true || gStepper_Obj.MANUAL_ADVANCE == true
            || gStepper_Obj.RECIPROCATION == true)
            && gStepper_Obj.MOTOR_CMD_ACTIVE2 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false)
    {
        // update steps to speed and steps to stop calculations
        // only calculate when the motor is idle and commanded to move
        time = (float) (DesiredTargetSpeed - StartingSpeed) / AccelRate;
        time_squared = time * time;
        // calculate steps to target
        Steps2Target = ((AccelRate >> 1) * (time_squared))
                + (StartingSpeed * time);
        // calculate Steps2Stop if:
        //  StepsToStop in GUI is >1 and
        //  StoppingSpeed is not equal Starting Speed
        //
        if ((StepsToStop == 2 && use_one_edge == 1) || StepsToStop == 1)
        {
            if (StartingSpeed != StoppingSpeed)
            {   // recalculate time if necessary
                time = (float) (DesiredTargetSpeed - StoppingSpeed) / AccelRate;
                time_squared = time * time;
            }
            // calculate steps required to stop
            Steps2Stop = ((AccelRate >> 1) * (time_squared))
                    + (StartingSpeed * time);
        }
        else    // defined number of steps to stop
        {
            Steps2Stop = StepsToStop;
        }
        // calculate stepping rate update (integer value divide by 32)
        if (AccelRate > 31 || AccelRate == 0)                                    // if >=32 or 0
        {
            SteppingRateUpdate = AccelRate >> 5;                                 // divide by 32 by right shifting 5 positions
        }
        else
        {
            SteppingRateUpdate = 1;                                              // set it to 1 to get correct value
        }

        if (StartingSpeed >= DesiredTargetSpeed)
        // Special case -- override prior settings
        // allow motor to attempt to run only at starting speed
        {
            SteppingRateUpdate = 0;
            Steps2Target = 0;
            Steps2Stop = 0;
        }
        // transition from ACCEL to DECEL when count equals (StepsToMove - Steps2Stop) <=0
        if ((Steps2Stop > (StepsToMove >> 1))
                && (StartingSpeed == StoppingSpeed))
            Steps2Decel = StepsToMove >> 1;                                      // decel at midpoint
        else
            Steps2Decel = StepsToMove - Steps2Stop;
    }

    // start running or stopping the motor if commanded and not manually advancing
    if (gStepper_Obj.RUN_STEPPER == true
            && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE2 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false)
    {
        // START STEPPER
        // case to allow 40 PPS
        if (Use2MHzTimerClk == false)
            TACTL = TASSEL_2 | ID_0 | MC_0;                                      // use 8MHz clock
        else
            TACTL = TASSEL_2 | ID_2 | MC_0;                                      // use 2MHz clock

        gStepper_Obj.MOTOR_CMD_ACTIVE1 = true;                                   // Indicate that the motor is running
        LastCommandWasRunSteps = true;                                           // flag to determine if the timer requires clearing
        DesiredStepperSpeed = DesiredTargetSpeed;                                // Configure the final stepping rate in PPS
        SteppingRate = StartingSpeed;                                            // This is the starting speed from the GUI

        // adjust for lower step rates
        if (Use2MHzTimerClk == false)
        {
            SteppingRateTMR = SMCLK_FREQ / StartingSpeed;                        // Convert here to the timer value -- xxTMR value to be loaded into the timer
        }

        else
        {
            SteppingRateTMR = SMCLK_FREQ_2MHz / StartingSpeed;                   // Convert here to the timer value -- xxTMR value to be loaded into the timer
        }


        gStepper_Obj.AccelerateState = ACCEL;                                    // signal to start state machine
        // Configure TimerA
        TACCTL1 &= ~OUTMOD_7;                                                    // Clear OUTMODx bits
        TACCTL1 |= OUTMOD_3;                                                     // Configure the timer as SET/RESET
        TACCTL0 |= CCIE;
        TACCR0 = SteppingRateTMR - 1;                                            // Start the timer here with initial value
        TACCR1 = SteppingRateTMR >> 1;                                           // set TBCCR1 to the midpoint of the TBCCR0 for toggle
        // configure TimerB
        TBCCTL1 &= ~OUTMOD_7;                                                    // Clear OUTMODx bits
        TBCCTL1 |= OUTMOD_3;                                                     // to SET/RESET
        TBCCTL0 |= CCIE;                                                         // TIMER_B0 dedicated interrupt
        TBCCR0 = TIME_32ms;                                                      // set timer to 32ms
        TBCCR1 = TIME_32ms >> 1;

        // only start timer if there is no stall fault and stop on stall is not set
        if (ok_to_start_timer == true)
        {
            TACTL |= MC_1;                                                       // Start timerA by selecting up mode
            TBCTL |= MC_1;                                                       // start timerB
        }

    }
    if ((gStepper_Obj.RUN_STEPPER == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE1 == true))
    // motor is running and stop is issued
    {
        // STOP STEPPER
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;                                  // must set low when stopping
        gStepper_Obj.AccelerateState = DECEL;
        DesiredStepperSpeed = StoppingSpeed;                                     // Speed to stop at
    }
    // MOVE_STEPS
    // Manually advance steps if commanded
    // only allow starting if not automatically running
    if (gStepper_Obj.MANUAL_ADVANCE == true
            && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE2 == false
            && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false)
    {
        gStepper_Obj.MOTOR_CMD_ACTIVE2 = true;
        if (LastCommandWasRunSteps == true || use_one_edge == false)             // added control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TACTL |= TACLR;                                                      // reset timerA
            // to allow 50 PPS
            if (Use2MHzTimerClk == false)
                TACTL = TASSEL_2 | ID_0 | MC_0;                                  // SMCLK/1, 8MHz
            else
                TACTL = TASSEL_2 | ID_2 | MC_0;                                  // SMCLK/4, 2MHz

            TACCTL1 &= ~OUTMOD_7;                                                // Clear OUTMODx bits
            TACCTL1 |= OUTMOD_3;                                                 // Configure the timer as SET/RESET. TBCCR0 does the reset
            TACCTL0 |= CCIE;
        }
        LastCommandWasRunSteps = false;                                          // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed = DesiredTargetSpeed;                                // Configure the Frequency Rate

        SteppingRate = StartingSpeed;
        if (Use2MHzTimerClk == false)
            SteppingRateTMR = SMCLK_FREQ / StartingSpeed;                        // Convert here to the timer value -- xxTMR
        // value to be loaded into the timer
        else
            SteppingRateTMR = SMCLK_FREQ_2MHz / StartingSpeed;                   // Convert here to the timer value -- xxTMR

        gStepper_Obj.AccelerateState = ACCEL;                                    // signal to start state machine
        tmpStepsToMove = 0;                                                      // clear counter
        MoveSteps = true;                                                        // set flag to indicate manual advance
        // configure Timer A
        TACCR0 = SteppingRateTMR - 1;                                            // start the timer
        TACCR1 = SteppingRateTMR >> 1;                                           // now using count up
        // configure TimerB
        TBCCTL1 &= ~OUTMOD_7;                                                    // Clear OUTMODx bits
        TBCCTL1 |= OUTMOD_3;                                                     // to SET/RESET for 32ms
        TBCCTL0 |= CCIE;                                                         // TIMER_B0 dedicated interrupt
        TBCCR0 = TIME_32ms;                                                      // set timer to 32ms
        TBCCR1 = TIME_32ms >> 1;
        // start timers
        TACTL |= MC_1;                                                           // Start timer by selecting up mode
        TBCTL |= MC_1;
    }

    if (gStepper_Obj.RECIPROCATION == true
            && gStepper_Obj.MANUAL_ADVANCE == true
            && temp_reciprocation == false)
    {
        last_reciprocation_state = 1;
        temp_reciprocation = true;
        gStepper_Obj.MOTOR_CMD_ACTIVE3 = true;
        if (LastCommandWasRunSteps == true || use_one_edge == false)             // control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TACTL |= TACLR;                                                      // reset timerA
            // to allow 50 PPS
            if (Use2MHzTimerClk == false)
                TACTL = TASSEL_2 | ID_1 | MC_0;
            else
                TACTL = TASSEL_2 | ID_2 | MC_0;

            TACCTL1 &= ~OUTMOD_7;                                                // Clear OUTMODx bits
            TACCTL1 |= OUTMOD_3;                                                 // Configure the timer as TOGGLE_RESET. TBCCR0 does the reset
            TACCTL0 |= CCIE;
        }
        LastCommandWasRunSteps = false;                                          // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed = DesiredTargetSpeed;                                // Configure the Frequency Rate

        SteppingRate = StartingSpeed;
        if (Use2MHzTimerClk == false)
            SteppingRateTMR = SMCLK_FREQ / StartingSpeed;                        // Convert here to the timer value -- xxTMR means the
        // value to be loaded into the timer
        else
            SteppingRateTMR = SMCLK_FREQ_2MHz / StartingSpeed;                   // Convert here to the timer value -- xxTMR means the
        __delay_cycles(10000000);
        gStepper_Obj.AccelerateState = ACCEL;                                    // signal to start state machine
        tmpStepsToMove = 0;                                                      // clear counter
        MoveSteps = true;                                                        // set flag to indicate manual advance
        // configure Timer A
        TACCR0 = SteppingRateTMR - 1;                                            // start the timer
        TACCR1 = SteppingRateTMR >> 1;                                           // now using count up
        // configure TimerB
        TBCCTL1 &= ~OUTMOD_7;                                                    // Clear OUTMODx bits
        TBCCTL1 |= OUTMOD_3;                                                     // to SET/RESET for 32ms
        TBCCTL0 |= CCIE;                                                         // TIMER_B0 dedicated interrupt
        TBCCR0 = TIME_32ms;                                                      // set timer to 32ms
        TBCCR1 = TIME_32ms >> 1;
        // start timers
        TACTL |= MC_1;                                                           // Start timer by selecting up mode
        TBCTL |= MC_1;
    }
    // Special case of updating speed during running
    // if the Desired Target Speed has been updated, then the stepper speed should be increased or decreased
    if ((gStepper_Obj.RUN_STEPPER == true)
            && (DesiredStepperSpeed != DesiredTargetSpeed))
    {
        if (DesiredTargetSpeed > DesiredStepperSpeed)
        {
            gStepper_Obj.AccelerateState = ACCEL;
        }
        else
            gStepper_Obj.AccelerateState = DECEL2NEWTARGET;
        // reset DesiredStepperSpeed
        DesiredStepperSpeed = DesiredTargetSpeed;
    }
    // Special case of stall fault while driving
    // reverse direction
    if (gStepper_Obj.StallState != DISABLED
            && (gStepper_Obj.REVERSE_ON_STALL == true)
            && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STALL_MASK))
    {
        RecoverFromStall = true;
        gStepper_Obj.AccelerateState = IDLE;
        last_stall_state = gStepper_Obj.StallState;
        if (gStepper_Obj.StallState == READY4STALL
                && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STL_LRN_OK_MASK))
        {
            gStepper_Obj.StallState = SUCCESS; // notify user stall learning was a success
        }
        else if (gStepper_Obj.StallState < SUCCESS)
            gStepper_Obj.StallState = BEGIN;
    }
    // stop on stall if reverse on stall is not set and stop on stall is set
    if (gStepper_Obj.StallState != DISABLED
            && ((gStepper_Obj.REVERSE_ON_STALL == false)
            && (gStepper_Obj.Active_STL_DET == false))
            && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STALL_MASK))
    {
        ok_to_start_timer = false;
        gStepper_Obj.AccelerateState = IDLE;
        // commands to signal stepping has halted
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
        gStepper_Obj.RUN_STEPPER = false;
        gStepper_Obj.MANUAL_ADVANCE = false;
        gStepper_Obj.RECIPROCATION = false;
        last_stall_state = gStepper_Obj.StallState;
        if (gStepper_Obj.StallState == READY4STALL && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STL_LRN_OK_MASK))
        {
            // set SUCCESS state on stop on stall
            gStepper_Obj.StallState = SUCCESS;             // stall learning was successfull
        }
    }
    else if (gStepper_Obj.StallState != DISABLED
            && ((gStepper_Obj.REVERSE_ON_STALL == false)
            && (gStepper_Obj.Active_STL_DET == true))
            && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STALL_MASK))
    {
        if (gStepper_Obj.StallState == READY4STALL && (SPI_readRegister(SPI_REG_DIAG_STATUS_2) & STL_LRN_OK_MASK))
         {
             // set SUCCESS state on stop on stall
             last_stall_state = gStepper_Obj.StallState;
             gStepper_Obj.StallState = SUCCESS;             // notify user stall learning was a success
         }

        // Clear Fault by CLR_FLT in CTRL4 Register
        gDeviceSpiReg.ctrl4_reg |= CLR_FLT_MASK;
        SPI_writeRegister(SPI_REG_CTRL4, (uint16_t) gDeviceSpiReg.ctrl4_reg);
        _delay_cycles(800);                                // 50us settling delay
        // End Clear Fault
    }
    else
        ok_to_start_timer = true;

    if (gStepper_Obj.StallState == DISABLED && last_stall_state != DISABLED)
    {
        gStepper_Obj.REVERSE_ON_STALL = false;
        gStepper_Obj.DISABLE_STALL_ACCEL = true;
        SPI_writeRegister(SPI_REG_CTRL5, (SPI_readRegister(SPI_REG_CTRL5) & DIS_STL_MASK));
        last_stall_state = DISABLED;
        SPI_writeRegister(SPI_REG_CTRL6, 3);           // set stall threshold to default value
        SPI_writeRegister(SPI_REG_CTRL7, (SPI_readRegister(SPI_REG_CTRL7) & 0xF0)); // set stall threshold to default value

        // Clear Fault by CLR_FLT in CTRL4 Register
        gDeviceSpiReg.ctrl4_reg |= CLR_FLT_MASK;
        SPI_writeRegister(SPI_REG_CTRL4, (uint16_t) gDeviceSpiReg.ctrl4_reg);
        _delay_cycles(800);                      // 50us settling delay
        // End Clear Fault
    }
}

// Stepper State machine
// This is used to move the motor
void MotorStateMachine()
{
    // update the SteppingRateTMR if required
    // reduce the number of calculations
    if (UpdateSteppingRateTMR == true)
    {
        // automatically select timer clock source based on stepping rate to allow very low as well as very high stepping rates
        if(SteppingRate > 399 && Use2MHzTimerClk == true)
            Use2MHzTimerClk = false;
        else if(SteppingRate < 400 && Use2MHzTimerClk == false)
            Use2MHzTimerClk = true;

        // recalculate stepping rate update in case Acceleration Rate was updated (integer value divide by 32)
        AccelRate = (unsigned long) gStepper_Obj.ACCEL_RATE << use_one_edge;     // Configure the acceleration rate in case Acceleration Rate was updated
        if (AccelRate > 31 || AccelRate == 0)                                    // if >=32 or 0
        {
            SteppingRateUpdate = AccelRate >> 5;                                 // divide by 32 by right shifting 5 positions
        }
        else
        {
            SteppingRateUpdate = 1;                                              // set it to 1 to get correct value
        }

        // setup Timer_A clock
        if (Use2MHzTimerClk == false)
        {
            TACTL &= ~ID_3;                                                      // use 8MHz clock
            SteppingRateTMR = SMCLK_FREQ / SteppingRate;                         // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        }
        else
        {
            TACTL &= ~ID_3;
            TACTL |= ID_2;                                                       // use 2MHz clock
            SteppingRateTMR = SMCLK_FREQ_2MHz / SteppingRate;                    // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        }

        UpdateSteppingRateTMR = false;                                           // clear flag
    }
    switch (gStepper_Obj.AccelerateState)
    {
    case (IDLE):
        if (gStepper_Obj.RECIPROCATION == false && gDRV8434_Obj.nSleep == 1
                && gDRV8434_Obj.gDRVEN == 1)
        {
            gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
            gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
            gStepper_Obj.MANUAL_ADVANCE = false;           // clear the flags once stops
            MoveSteps = false;                             // clear the MoveSteps flag here
            // reset the timers if the last command was run motor
            if (LastCommandWasRunSteps == true)
            {
                TACTL &= ~MC_3;                            // Stop Timer_A
                TACCTL1 &= ~(OUTMOD_7 + OUT);              // Output low
                TACTL |= TACLR;
                TBCTL &= ~MC_3;                            // Stop Timer_B
                TBCTL |= TACLR;
            }

            if (RecoverFromStall == true)                  // after stall fault, wait a little bit, change direction, them begin driving again
            {
                __delay_cycles(10000000);
                if (gStepper_Obj.gDIR == 1)                // change direction
                {
                    gStepper_Obj.gDIR = 0;
                }
                else
                    gStepper_Obj.gDIR = 1;

                RecoverFromStall = false;
                // Clear Fault by CLR_FLT in CTRL4 Register
                gDeviceSpiReg.ctrl4_reg |= CLR_FLT_MASK;
                SPI_writeRegister(SPI_REG_CTRL4, (uint16_t) gDeviceSpiReg.ctrl4_reg);
                _delay_cycles(800);                        // 50us settling delay
                // End Clear Fault
                gStepper_Obj.AccelerateState = ACCEL;
            }
            if ((gStepper_Obj.RECIPROCATION == true
                    && gStepper_Obj.MANUAL_ADVANCE == true))
            {
                __delay_cycles(10000000);                  // pause here if reciprocating
            }

        }
        break;
    case (ACCEL):
        {
            // do nothing in ACCEL mode, wait for external input to change state
            if (gStepper_Obj.StallState >= 4 && gStepper_Obj.DISABLE_STALL_ACCEL
                    && gDeviceSpiReg.ctrl5_reg & EN_STL_MASK && SteppingRate != DesiredTargetSpeed)   // Disable stall during acceleration
            {
                SPI_writeRegister(SPI_REG_CTRL5, ((SPI_readRegister(SPI_REG_CTRL5) & DIS_STL_MASK))); // Disable Stall detection
            }

            stall_learn_number_of_steps = 0;               // set step counter to 0 during acceleration
        }
        break;
    case (DECEL):
        {
            // do nothing in DECEL mode, wait for external input to change state
        }
        break;
    case (RUN):
        {
            if ((gStepper_Obj.StallState >= 4) && !(gDeviceSpiReg.ctrl5_reg & EN_STL_MASK)) // Enable stall detection if Initiate Stall is enabled
                {
                    SPI_writeRegister(SPI_REG_CTRL5, (SPI_readRegister(SPI_REG_CTRL5) | EN_STL_MASK)); // Enable Stall Detection
                    stall_learn_number_of_steps = 0;               // set step counter to 0 during acceleration
                }
        }
        break;
    case (DECEL2NEWTARGET):
        {
            // do nothing in DECEL mode, wait for external input to change state
        }
        break;
    }
}

void DetermineMicrostepping(char CTRL3Data)
{
    MICROSTEP_MODE_BITS = CTRL3Data & 0x0F;
    switch (CTRL3Data & 0x0F)
    {
    case (0x00):                                           // Full step (2-phase excitation) with 100% current
    case (0x01):                                           // Full step (2-phase excitation) with 71% current
    {
        stall_learn_microstep_divider = 2;
    }
        break;
    case (0x02):                                           // Non-circular 1/2 step
    case (0x03):                                           // 1/2 step
    {
        stall_learn_microstep_divider = 3;
    }
        break;
    default:                                               // 1/4 step to 1/32 step
    {
        stall_learn_microstep_divider = CTRL3Data & 0x0F;
    }
    }
}

// Analog TRQ_COUNT output
void UpdateStallInfo()
{
    // disable SPI actions when asleep or outputs disabled
    if (gDRV8434_Obj.nSleep == 1)
    {
        // read the TRQ_COUNT register and write to DAC0
        DAC12_0DAT = (SPI_readRegister((uint8_t) SPI_REG_CTRL9 & TRQ_COUNT_MASK_MSB) << 8) + SPI_readRegister((uint8_t) SPI_REG_CTRL8);
    }
}

void SetMotorParameters()
{
    if (UpdateMotorParameters == true)
    {
        // increment or decrement to the new timer A value
        if (gStepper_Obj.AccelerateState == ACCEL
                && (SteppingRate <= DesiredTargetSpeed))
        {
            // increase pps and calculate new pps
            if ((SteppingRate + SteppingRateUpdate) >= DesiredTargetSpeed)
            {
                gStepper_Obj.AccelerateState = RUN;        // No longer accelerate
                SteppingRate = DesiredTargetSpeed;
            }
            else
            {
                SteppingRate += SteppingRateUpdate;        // Increment
                gStepper_Obj.AccelerateState = ACCEL;
            }
        }
        if (gStepper_Obj.AccelerateState == DECEL)
        {
            if (SteppingRate >= StoppingSpeed)
            {
                if (SteppingRate <= (StoppingSpeed + SteppingRateUpdate))
                {
                    if (MoveSteps == false && SteppingRate == StoppingSpeed)
                        gStepper_Obj.AccelerateState = IDLE; // Stop motor when speed is reached
                    SteppingRate = StoppingSpeed;
                }
                else
                {
                    SteppingRate -= SteppingRateUpdate;    // Decrement
                    gStepper_Obj.AccelerateState = DECEL;
                }
            }
            else // already less than stopping speed
            {
                if (MoveSteps == false)
                    gStepper_Obj.AccelerateState = IDLE;   // Stop motor when speed is reached
            }

        }
        // this allows for the motor to spin down to new speed without stopping
        if (gStepper_Obj.AccelerateState == DECEL2NEWTARGET)
        {
            if (SteppingRate >= DesiredTargetSpeed)
            {
                // decrease pps and calculate new pps
                if ((SteppingRate - SteppingRateUpdate) <= DesiredTargetSpeed)
                {
                    if (SteppingRate == DesiredTargetSpeed)
                    {
                        gStepper_Obj.AccelerateState = RUN;// switch to RUN when target speed is reached
                    }
                    SteppingRate = DesiredTargetSpeed;
                }
                else
                {
                    SteppingRate -= SteppingRateUpdate;    // Decrement
                    gStepper_Obj.AccelerateState = DECEL2NEWTARGET;
                }
            }
        }
        // do not perform the division here -- only update the stepping rate
        // if the stepping rate has been updated, set a flag
        if (SteppingRate != PriorSteppingRate)
        {
            UpdateSteppingRateTMR = true;                  // Set flag to re-calculate stepping rate TMR
        }
        UpdateMotorParameters = false;                     // clear the flag
    }
}

void DetermineNextAction()
{
    if (UpdateNextAction == true)
    {
        if (MoveSteps)                                     // if manual advance count steps
        {                                                  // when complete
            if (tmpStepsToMove >= Steps2Decel)
            {
                gStepper_Obj.AccelerateState = DECEL;      // start deceleration
            }
            if (tmpStepsToMove >= StepsToMove)             // stop here when equal
            {
                gStepper_Obj.AccelerateState = IDLE;
                temp_reciprocation = false;
                if ((gStepper_Obj.RECIPROCATION == true
                        && gStepper_Obj.MANUAL_ADVANCE == true))
                {
                    tmpStepsToMove = 0;                    // clear counter
                    gStepper_Obj.gDIR = nextDIR;
                }
                if (gStepper_Obj.RECIPROCATION == false)
                {
                    if (last_reciprocation_state == 1)
                    {
                        gStepper_Obj.gDIR = nextDIR;
                        last_reciprocation_state = 0;
                    }
                    MoveSteps = false;
                }

                TACTL &= ~MC_3;                            // Stop Timer_A
                TACCTL1 &= ~(OUTMOD_7 + OUT);              // Output low
                TACTL |= TACLR;
                TBCTL &= ~MC_3;                            // Stop Timer_B
                TBCTL |= TACLR;
            }
        }
        if (last_stall_state |= gStepper_Obj.StallState)
            last_stall_state = gStepper_Obj.StallState;
        if (gStepper_Obj.StallState == LEARNING)
        {
            stall_learn_number_of_steps++;                 // Count steps
            if ((stall_learn_number_of_steps >> stall_learn_microstep_divider)
                    > 16)
            {
                SPI_writeRegister(SPI_REG_CTRL6, 3);       // set stall threshold to default value
                SPI_writeRegister(SPI_REG_CTRL7, (SPI_readRegister(SPI_REG_CTRL7) & 0xF0)); // set stall threshold to default value
                SPI_writeRegister(SPI_REG_CTRL5, ((SPI_readRegister(SPI_REG_CTRL5) | (stall_learn_report_data)))); // set STL_LRN bit to begin learning stall
                last_stall_state = gStepper_Obj.StallState;
                gStepper_Obj.StallState = READY4STALL;     // If number of stall learning steps is reached, tell the user to stall the motor
                stall_learn_number_of_steps = 0;           // set step counter to 0
            }
        }
        if (gStepper_Obj.SPI_STEP_DIR_EN)
            write_spi_step_pulse = true;
        UpdateNextAction = false;                          // clear the flag
    }
}
